@@ -2,11 +2,12 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "image.h"
-#include "../../../common/sys/string.h"
+#include "../../../common/sys/estring.h"
 
 #include <map>
 #include <iostream>
 
+
 namespace embree
 {
   double compareImages(Ref<Image> image0, Ref<Image> image1)
@@ -56,6 +57,12 @@ namespace embree
     if (ext == "png" ) return loadOIIO(fileName);
     if (ext == "jpg" ) return loadOIIO(fileName);
 #endif
+
+    if (ext == "bmp" ) return loadSTB(fileName);
+    if (ext == "png" ) return loadSTB(fileName);
+    if (ext == "jpg" ) return loadSTB(fileName);
+
+    if (ext == "exr" ) return loadEXR(fileName);
     
     if (ext == "pfm" ) return loadPFM(fileName);
     if (ext == "ppm" ) return loadPPM(fileName);
@@ -96,6 +103,12 @@ namespace embree
     if (ext == "jpg" ) { storeOIIO(img, fileName);  return; }
 #endif
 
+    if (ext == "bmp" ) { storeSTB(img, fileName);  return; }
+    if (ext == "png" ) { storeSTB(img, fileName);  return; }
+    if (ext == "jpg" ) { storeSTB(img, fileName);  return; }
+
+    if (ext == "exr" ) { storeEXR(img, fileName);  return; }
+
     if (ext == "pfm" ) { storePFM(img, fileName);  return; }
     if (ext == "ppm" ) { storePPM(img, fileName);  return; }
     if (ext == "tga" ) { storeTga(img, fileName);  return; }
