@@ -1,26 +1,16 @@
 // Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
-#include "../common/tutorial/tutorial_device.h"
+#include "dynamic_scene_device.h"
 
 namespace embree {
 
-#if 0
-const int numSpheres = 1000;
-const int numPhi = 5;
-#else
-const int numSpheres = 20;
-const int numPhi = 120;
-//const int numPhi = 256;
-#endif
-const int numTheta = 2*numPhi;
+/* all features required by this tutorial */
+#define FEATURE_MASK \
+  RTC_FEATURE_FLAG_TRIANGLE
 
-/* scene data */
 RTCScene g_scene = nullptr;
-Vec3fa position[numSpheres];
-Vec3fa colors[numSpheres+1];
-float radius[numSpheres];
-int disabledID = -1;
+TutorialData data;
 
 /* adds a sphere to the scene */
 unsigned int createSphere (RTCBuildQuality quality, const Vec3fa& pos, const float r)
@@ -30,32 +20,32 @@ unsigned int createSphere (RTCBuildQuality quality, const Vec3fa& pos, const flo
   rtcSetGeometryBuildQuality(geom, quality);
 
   /* map triangle and vertex buffer */
-  Vertex*   vertices  = (Vertex*  ) rtcSetNewGeometryBuffer(geom,RTC_BUFFER_TYPE_VERTEX,0,RTC_FORMAT_FLOAT3,sizeof(Vertex),numTheta*(numPhi+1));
-  Triangle* triangles = (Triangle*) rtcSetNewGeometryBuffer(geom,RTC_BUFFER_TYPE_INDEX,0,RTC_FORMAT_UINT3,sizeof(Triangle),2*numTheta*(numPhi-1));
+  Vertex*   vertices  = (Vertex*  ) rtcSetNewGeometryBuffer(geom,RTC_BUFFER_TYPE_VERTEX,0,RTC_FORMAT_FLOAT3,sizeof(Vertex),data.numTheta*(data.numPhi+1));
+  Triangle* triangles = (Triangle*) rtcSetNewGeometryBuffer(geom,RTC_BUFFER_TYPE_INDEX,0,RTC_FORMAT_UINT3,sizeof(Triangle),2*data.numTheta*(data.numPhi-1));
 
   /* create sphere geometry */
   int tri = 0;
-  const float rcpNumTheta = rcp((float)numTheta);
-  const float rcpNumPhi   = rcp((float)numPhi);
-  for (int phi=0; phi<=numPhi; phi++)
+  const float rcpNumTheta = rcp((float)data.numTheta);
+  const float rcpNumPhi   = rcp((float)data.numPhi);
+  for (int phi=0; phi<=data.numPhi; phi++)
   {
-    for (int theta=0; theta<numTheta; theta++)
+    for (int theta=0; theta<data.numTheta; theta++)
     {
       const float phif   = phi*float(pi)*rcpNumPhi;
       const float thetaf = theta*2.0f*float(pi)*rcpNumTheta;
-      Vertex& v = vertices[phi*numTheta+theta];
+      Vertex& v = vertices[phi*data.numTheta+theta];
       v.x = pos.x + r*sin(phif)*sin(thetaf);
       v.y = pos.y + r*cos(phif);
       v.z = pos.z + r*sin(phif)*cos(thetaf);
     }
     if (phi == 0) continue;
 
-    for (int theta=1; theta<=numTheta; theta++)
+    for (int theta=1; theta<=data.numTheta; theta++)
     {
-      int p00 = (phi-1)*numTheta+theta-1;
-      int p01 = (phi-1)*numTheta+theta%numTheta;
-      int p10 = phi*numTheta+theta-1;
-      int p11 = phi*numTheta+theta%numTheta;
+      int p00 = (phi-1)*data.numTheta+theta-1;
+      int p01 = (phi-1)*data.numTheta+theta%data.numTheta;
+      int p10 = phi*data.numTheta+theta-1;
+      int p11 = phi*data.numTheta+theta%data.numTheta;
 
       if (phi > 1) {
         triangles[tri].v0 = p10;
@@ -64,7 +54,7 @@ unsigned int createSphere (RTCBuildQuality quality, const Vec3fa& pos, const flo
         tri++;
       }
 
-      if (phi < numPhi) {
+      if (phi < data.numPhi) {
         triangles[tri].v0 = p11;
         triangles[tri].v1 = p01;
         triangles[tri].v2 = p10;
@@ -74,7 +64,7 @@ unsigned int createSphere (RTCBuildQuality quality, const Vec3fa& pos, const flo
   }
 
   rtcCommitGeometry(geom);
-  unsigned int geomID = rtcAttachGeometry(g_scene,geom);
+  unsigned int geomID = rtcAttachGeometry(data.g_scene,geom);
   rtcReleaseGeometry(geom);
   return geomID;
 }
@@ -106,34 +96,36 @@ unsigned int addGroundPlane (RTCScene scene_i)
 /* called by the C++ code for initialization */
 extern "C" void device_init (char* cfg)
 {
+  TutorialData_Constructor(&data);
+  
   /* create scene */
-  g_scene = rtcNewScene(g_device);
-  rtcSetSceneFlags(g_scene,RTC_SCENE_FLAG_DYNAMIC | RTC_SCENE_FLAG_ROBUST);
-  rtcSetSceneBuildQuality(g_scene,RTC_BUILD_QUALITY_LOW);
+  data.g_scene = g_scene = rtcNewScene(g_device);
+  rtcSetSceneFlags(data.g_scene,RTC_SCENE_FLAG_DYNAMIC | RTC_SCENE_FLAG_ROBUST);
+  rtcSetSceneBuildQuality(data.g_scene,RTC_BUILD_QUALITY_LOW);
 
   /* create some triangulated spheres */
-  for (int i=0; i<numSpheres; i++)
+  for (int i=0; i<data.numSpheres; i++)
   {
-    const float phi = i*2.0f*float(pi)/numSpheres;
-    const float r = 2.0f*float(pi)/numSpheres;
+    const float phi = i*2.0f*float(pi)/data.numSpheres;
+    const float r = 2.0f*float(pi)/data.numSpheres;
     const Vec3fa p = 2.0f*Vec3fa(sin(phi),0.0f,-cos(phi));
     //RTCBuildQuality quality = i%3 == 0 ? RTC_BUILD_QUALITY_MEDIUM : i%3 == 1 ? RTC_BUILD_QUALITY_REFIT : RTC_BUILD_QUALITY_LOW;
     RTCBuildQuality quality = i%2 ? RTC_BUILD_QUALITY_REFIT : RTC_BUILD_QUALITY_LOW;
     //RTCBuildQuality quality = RTC_BUILD_QUALITY_REFIT;
     int id = createSphere(quality,p,r);
-    position[id] = p;
-    radius[id] = r;
-    colors[id].x = (i%16+1)/17.0f;
-    colors[id].y = (i%8+1)/9.0f;
-    colors[id].z = (i%4+1)/5.0f;
+    data.position[id] = Vec3fa(p);
+    data.radius[id] = r;
+    data.colors[id].x = (i%16+1)/17.0f;
+    data.colors[id].y = (i%8+1)/9.0f;
+    data.colors[id].z = (i%4+1)/5.0f;
   }
 
   /* add ground plane to scene */
-  int id = addGroundPlane(g_scene);
-  colors[id] = Vec3fa(1.0f,1.0f,1.0f);
+  int id = addGroundPlane(data.g_scene);
+  data.colors[id] = Vec3fa(1.0f,1.0f,1.0f);
 
   /* commit changes to scene */
-  rtcCommitScene (g_scene);
+  rtcCommitScene (data.g_scene);
 }
 
 /* animates the sphere */
@@ -145,9 +137,9 @@ void animateSphere (int taskIndex, int threadIndex, Vertex* vertices,
                          const float f)
 {
   int phi = taskIndex;
-  for (unsigned int theta=0; theta<numTheta; theta++)
+  for (unsigned int theta=0; theta<data.numTheta; theta++)
   {
-    Vertex* v = &vertices[phi*numTheta+theta];
+    Vertex* v = &vertices[phi*data.numTheta+theta];
     const float phif   = phi*float(pi)*rcpNumPhi;
     const float thetaf = theta*2.0f*float(pi)*rcpNumTheta;
     v->x = pos.x + r*sin(f*phif)*sin(thetaf);
@@ -157,22 +149,30 @@ void animateSphere (int taskIndex, int threadIndex, Vertex* vertices,
 }
 
 /* task that renders a single screen tile */
-Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats& stats)
+void renderPixelStandard(const TutorialData& data,
+                         int x, int y, 
+                         int* pixels,
+                         const unsigned int width,
+                         const unsigned int height,
+                         const float time,
+                         const ISPCCamera& camera, RayStats& stats)
 {
   /* initialize ray */
   Ray ray(Vec3fa(camera.xfm.p), Vec3fa(normalize(x*camera.xfm.l.vx + y*camera.xfm.l.vy + camera.xfm.l.vz)), 0.0f, inf);
 
   /* intersect ray with scene */
-  RTCIntersectContext context;
-  rtcInitIntersectContext(&context);
-  rtcIntersect1(g_scene,&context,RTCRayHit_(ray));
+  RTCIntersectArguments iargs;
+  rtcInitIntersectArguments(&iargs);
+  iargs.feature_mask = (RTCFeatureFlags) (FEATURE_MASK);
+  
+  rtcIntersect1(data.g_scene,RTCRayHit_(ray),&iargs);
   RayStats_addRay(stats);
 
   /* shade pixels */
   Vec3fa color = Vec3fa(0.0f);
   if (ray.geomID != RTC_INVALID_GEOMETRY_ID)
   {
-    Vec3fa diffuse = colors[ray.geomID];
+    Vec3fa diffuse = Vec3fa(data.colors[ray.geomID]);
     color = color + diffuse*0.1f;
     Vec3fa lightDir = normalize(Vec3fa(-1,-1,-1));
 
@@ -180,14 +180,22 @@ Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats&
     Ray shadow(ray.org + ray.tfar*ray.dir, neg(lightDir), 0.001f, inf);
 
     /* trace shadow ray */
-    rtcOccluded1(g_scene,&context,RTCRay_(shadow));
+    RTCOccludedArguments sargs;
+    rtcInitOccludedArguments(&sargs);
+    sargs.feature_mask = (RTCFeatureFlags) (FEATURE_MASK);
+    rtcOccluded1(data.g_scene,RTCRay_(shadow),&sargs);
     RayStats_addShadowRay(stats);
 
     /* add light contribution */
     if (shadow.tfar >= 0.0f)
       color = color + diffuse*clamp(-dot(lightDir,normalize(ray.Ng)),0.0f,1.0f);
   }
-  return color;
+
+  /* write color to framebuffer */
+  unsigned int r = (unsigned int) (255.0f * clamp(color.x,0.0f,1.0f));
+  unsigned int g = (unsigned int) (255.0f * clamp(color.y,0.0f,1.0f));
+  unsigned int b = (unsigned int) (255.0f * clamp(color.z,0.0f,1.0f));
+  pixels[y*width+x] = (b << 16) + (g << 8) + r;
 }
 
 /* renders a single screen tile */
@@ -210,14 +218,7 @@ void renderTileStandard(int taskIndex,
 
   for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
   {
-    /* calculate pixel color */
-    Vec3fa color = renderPixelStandard((float)x,(float)y,camera,g_stats[threadIndex]);
-
-    /* write color to framebuffer */
-    unsigned int r = (unsigned int) (255.0f * clamp(color.x,0.0f,1.0f));
-    unsigned int g = (unsigned int) (255.0f * clamp(color.y,0.0f,1.0f));
-    unsigned int b = (unsigned int) (255.0f * clamp(color.z,0.0f,1.0f));
-    pixels[y*width+x] = (b << 16) + (g << 8) + r;
+    renderPixelStandard(data,x,y,pixels,width,height,time,camera,g_stats[threadIndex]);
   }
 }
 
@@ -237,25 +238,25 @@ void renderTileTask (int taskIndex, int threadIndex, int* pixels,
 void animateSphere (int id, float time)
 {
   /* animate vertices */
-  RTCGeometry geom = rtcGetGeometry(g_scene,id);
+  RTCGeometry geom = rtcGetGeometry(data.g_scene,id);
   Vertex* vertices = (Vertex*) rtcGetGeometryBufferData(geom,RTC_BUFFER_TYPE_VERTEX,0);
-  const float rcpNumTheta = rcp((float)numTheta);
-  const float rcpNumPhi   = rcp((float)numPhi);
-  const Vec3fa pos = position[id];
-  const float r = radius[id];
+  const float rcpNumTheta = rcp((float)data.numTheta);
+  const float rcpNumPhi   = rcp((float)data.numPhi);
+  const Vec3fa pos = Vec3fa(data.position[id]);
+  const float r = data.radius[id];
   const float f = 2.0f*(1.0f+0.5f*sin(time));
 
   /* loop over all vertices */
-#if 1 // enables parallel execution
-  parallel_for(size_t(0),size_t(numPhi+1),[&](const range<size_t>& range) {
+#if !defined(EMBREE_SYCL_TUTORIAL) // enables parallel execution
+  parallel_for(size_t(0),size_t(data.numPhi+1),[&](const range<size_t>& range) {
     const int threadIndex = (int)TaskScheduler::threadIndex();
     for (size_t i=range.begin(); i<range.end(); i++)
       animateSphere((int)i,threadIndex,vertices,rcpNumTheta,rcpNumPhi,pos,r,f);
   }); 
 #else
-  for (unsigned int phi=0; phi<numPhi+1; phi++) for (int theta=0; theta<numTheta; theta++)
+  for (unsigned int phi=0; phi<data.numPhi+1; phi++) for (int theta=0; theta<data.numTheta; theta++)
   {
-    Vertex* v = &vertices[phi*numTheta+theta];
+    Vertex* v = &vertices[phi*data.numTheta+theta];
     const float phif   = phi*float(pi)*rcpNumPhi;
     const float thetaf = theta*2.0f*float(pi)*rcpNumTheta;
     v->x = pos.x+r*sin(f*phif)*sin(thetaf);
@@ -276,6 +277,25 @@ extern "C" void renderFrameStandard (int* pixels,
                           const ISPCCamera& camera)
 {
   /* render all pixels */
+#if defined(EMBREE_SYCL_TUTORIAL) && !defined(EMBREE_SYCL_RT_SIMULATION)
+  TutorialData ldata = data;
+  sycl::event event = global_gpu_queue->submit([=](sycl::handler& cgh){
+    const sycl::nd_range<2> nd_range = make_nd_range(height,width);
+    cgh.parallel_for(nd_range,[=](sycl::nd_item<2> item) {
+      const unsigned int x = item.get_global_id(1); if (x >= width ) return;
+      const unsigned int y = item.get_global_id(0); if (y >= height) return;
+      RayStats stats;
+      renderPixelStandard(ldata,x,y,pixels,width,height,time,camera,stats);
+    });
+  });
+  global_gpu_queue->wait_and_throw();
+
+  const auto t0 = event.template get_profiling_info<sycl::info::event_profiling::command_start>();
+  const auto t1 = event.template get_profiling_info<sycl::info::event_profiling::command_end>();
+  const double dt = (t1-t0)*1E-9;
+  ((ISPCCamera*)&camera)->render_time = dt;
+  
+#else
   const int numTilesX = (width +TILE_SIZE_X-1)/TILE_SIZE_X;
   const int numTilesY = (height+TILE_SIZE_Y-1)/TILE_SIZE_Y;
   parallel_for(size_t(0),size_t(numTilesX*numTilesY),[&](const range<size_t>& range) {
@@ -283,6 +303,7 @@ extern "C" void renderFrameStandard (int* pixels,
     for (size_t i=range.begin(); i<range.end(); i++)
       renderTileTask((int)i,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
   }); 
+#endif
 }
 
 /* called by the C++ code to render */
@@ -293,17 +314,17 @@ extern "C" void device_render (int* pixels,
                            const ISPCCamera& camera)
 {
   /* animate sphere */
-  for (int i=0; i<numSpheres; i++)
+  for (int i=0; i<data.numSpheres; i++)
     animateSphere(i,time+i);
 
   /* commit changes to scene */
-  rtcCommitScene (g_scene);
+  rtcCommitScene (data.g_scene);
 }
 
 /* called by the C++ code for cleanup */
 extern "C" void device_cleanup ()
 {
-  rtcReleaseScene (g_scene); g_scene = nullptr;
+  TutorialData_Destructor(&data);
 }
 
 } // namespace embree
