@@ -2,40 +2,25 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/math/random_sampler.h"
-#include "../common/tutorial/tutorial_device.h"
-#include "../common/tutorial/scene_device.h"
+// #include "../common/tutorial/tutorial_device.h"
+// #include "../common/tutorial/scene_device.h"
 #include "../common/tutorial/optics.h"
+#include "hair_geometry_device.h"
 
 namespace embree {
 
-/* accumulation buffer */
-Vec3ff* g_accu = nullptr;
-unsigned int g_accu_width = 0;
-unsigned int g_accu_height = 0;
-unsigned int g_accu_count = 0;
-Vec3fa g_accu_vx;
-Vec3fa g_accu_vy;
-Vec3fa g_accu_vz;
-Vec3fa g_accu_p;
-extern "C" bool g_changed;
-bool g_subdiv_mode = false;
-
-/* light settings */
-extern "C" Vec3fa g_dirlight_direction;
-extern "C" Vec3fa g_dirlight_intensity;
-extern "C" Vec3fa g_ambient_intensity;
-
-/* hair material */
-Vec3fa hair_K;
-Vec3fa hair_dK;
-Vec3fa hair_Kr;    //!< reflectivity of hair
-Vec3fa hair_Kt;    //!< transparency of hair
-
-void occlusionFilter(const RTCFilterFunctionNArguments* args);
+/* all features required by this tutorial */
+#define FEATURE_MASK \
+  RTC_FEATURE_FLAG_TRIANGLE | \
+  RTC_FEATURE_FLAG_ROUND_LINEAR_CURVE | \
+  RTC_FEATURE_FLAG_FILTER_FUNCTION_IN_ARGUMENTS
 
 /* scene data */
-extern "C" ISPCScene* g_ispc_scene;
 RTCScene g_scene = nullptr;
+TutorialData data;
+extern "C" bool g_changed;
+
+RTC_SYCL_INDIRECTLY_CALLABLE void occlusionFilter(const RTCFilterFunctionNArguments* args);
 
 void convertTriangleMesh(ISPCTriangleMesh* mesh, RTCScene scene_out)
 {
@@ -44,7 +29,7 @@ void convertTriangleMesh(ISPCTriangleMesh* mesh, RTCScene scene_out)
     rtcSetSharedGeometryBuffer(geom,RTC_BUFFER_TYPE_VERTEX,t,RTC_FORMAT_FLOAT3,mesh->positions[t],0,sizeof(Vertex),mesh->numVertices);
   }
   rtcSetSharedGeometryBuffer(geom,RTC_BUFFER_TYPE_INDEX,0,RTC_FORMAT_UINT3,mesh->triangles,0,sizeof(ISPCTriangle),mesh->numTriangles);
-  rtcSetGeometryOccludedFilterFunction(geom,occlusionFilter);
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom,false);
   rtcCommitGeometry(geom);
   rtcAttachGeometry(scene_out,geom);
   rtcReleaseGeometry(geom);
@@ -59,8 +44,8 @@ void convertHairSet(ISPCHairSet* hair, RTCScene scene_out)
   rtcSetSharedGeometryBuffer(geom,RTC_BUFFER_TYPE_INDEX,0,RTC_FORMAT_UINT,hair->hairs,0,sizeof(ISPCHair),hair->numHairs);
   if (hair->flags)
     rtcSetSharedGeometryBuffer(geom,RTC_BUFFER_TYPE_FLAGS,0,RTC_FORMAT_UCHAR,hair->flags,0,sizeof(char),hair->numHairs);
-  rtcSetGeometryOccludedFilterFunction(geom,occlusionFilter);
   rtcSetGeometryTessellationRate(geom,(float)hair->tessellation_rate);
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom,true);
   rtcCommitGeometry(geom);
   rtcAttachGeometry(scene_out,geom);
   rtcReleaseGeometry(geom);
@@ -198,7 +183,7 @@ typedef Vec3fa* uniform_Vec3fa_ptr;
 inline Vec3fa evalBezier(const unsigned int geomID, const unsigned int primID, const float t)
 {
   const float t0 = 1.0f - t, t1 = t;
-  const ISPCHairSet* hair = (const ISPCHairSet*) g_ispc_scene->geometries[geomID];
+  const ISPCHairSet* hair = (const ISPCHairSet*) data.ispc_scene->geometries[geomID];
   const Vec3fa* vertices = hair->positions[0];
   const ISPCHair* hairs = hair->hairs;
 
@@ -220,37 +205,36 @@ inline Vec3fa evalBezier(const unsigned int geomID, const unsigned int primID, c
 }
 
 /* occlusion filter function */
-void occlusionFilter(const RTCFilterFunctionNArguments* args)
+RTC_SYCL_INDIRECTLY_CALLABLE void occlusionFilter(const RTCFilterFunctionNArguments* args)
 
 {
-  IntersectContext* context = (IntersectContext*) args->context;
+  RayQueryContext* context = (RayQueryContext*) args->context;
+  const TutorialData* data = (const TutorialData*) context->tutorialData;
   Vec3fa* transparency = (Vec3fa*) context->userRayExt;
   if (!transparency) return;
     
   int* valid_i = args->valid;
-  struct RTCHitN* hit = args->hit;
-  const unsigned int N = args->N;
-  assert(N == 1);
+  //struct RTCHitN* hit = args->hit;
+  //const unsigned int N = args->N;
+  //assert(N == 1);
   bool valid = *((int*) valid_i);
   if (!valid) return;
  
   /* make all surfaces opaque */
-  unsigned int geomID = RTCHitN_geomID(hit,N,0);
-  ISPCGeometry* geometry = g_ispc_scene->geometries[geomID];
+  /*unsigned int geomID = RTCHitN_geomID(hit,N,0);
+  ISPCGeometry* geometry = data->ispc_scene->geometries[geomID];
   if (geometry->type == TRIANGLE_MESH) {
     *transparency = Vec3fa(0.0f);
     return;
-  }
-  Vec3fa T = hair_Kt;
+  }*/
+  Vec3fa T = data->hair_Kt;
   T = T * *transparency;
   *transparency = T;
-  if (eq(T,Vec3fa(0.0f)))
-    ;
-  else
+  if (max(T.x,max(T.y,T.z)) > 0.02f)
     valid_i[0] = 0;
 }
 
-Vec3fa occluded(RTCScene scene, IntersectContext* context, Ray& ray)
+Vec3fa occluded(RTCScene scene, RayQueryContext* context, Ray& ray)
 {
   Vec3fa transparency = Vec3fa(1.0f);
   context->userRayExt = &transparency;
@@ -258,22 +242,37 @@ Vec3fa occluded(RTCScene scene, IntersectContext* context, Ray& ray)
   ray.geomID = RTC_INVALID_GEOMETRY_ID;
   ray.primID = RTC_INVALID_GEOMETRY_ID;
   ray.mask = -1;
-  rtcOccluded1(scene,&context->context,RTCRay_(ray));
 
-  return transparency;
+  RTCOccludedArguments args;
+  rtcInitOccludedArguments(&args);
+  args.context = &context->context;
+  args.feature_mask = (RTCFeatureFlags) (FEATURE_MASK);
+  args.filter = occlusionFilter;
+  
+  rtcOccluded1(scene,RTCRay_(ray),&args);
+  context->userRayExt = NULL;
+
+  if (ray.tfar < 0) return Vec3fa(0.0f);
+  else              return transparency;
 }
 
 /* task that renders a single screen tile */
-Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats& stats)
+Vec3fa renderPixel(const TutorialData& data, float x, float y, const ISPCCamera& camera, RayStats& stats)
 {
   RandomSampler sampler;
-  RandomSampler_init(sampler, (int)x, (int)y, g_accu_count);
+  RandomSampler_init(sampler, (int)x, (int)y, data.accu_count);
   x += RandomSampler_get1D(sampler);
   y += RandomSampler_get1D(sampler);
   float time = RandomSampler_get1D(sampler);
 
-  IntersectContext context;
+  RayQueryContext context;
   InitIntersectionContext(&context);
+  context.tutorialData = (void*) &data;
+
+  RTCIntersectArguments args;
+  rtcInitIntersectArguments(&args);
+  args.context = &context.context;
+  args.feature_mask = (RTCFeatureFlags) (FEATURE_MASK);
   
   /* initialize ray */
   Ray ray(Vec3fa(camera.xfm.p), Vec3fa(normalize(x*camera.xfm.l.vx + y*camera.xfm.l.vy + camera.xfm.l.vz)), 0.0f, inf);
@@ -285,22 +284,22 @@ Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats&
   while (true)
   {
     /* terminate ray path */
-    if (reduce_max(weight) < 0.01 || depth > 20)
+    if (reduce_max(weight) < 0.01f || depth > 20)
       return color;
 
     /* intersect ray with scene and gather all hits */
-    rtcIntersect1(g_scene,&context.context,RTCRayHit_(ray));
+    rtcIntersect1(data.scene,RTCRayHit_(ray),&args);
     RayStats_addRay(stats);
 
     /* exit if we hit environment */
     if (ray.geomID == RTC_INVALID_GEOMETRY_ID)
-      return color + weight*Vec3fa(g_ambient_intensity);
+      return color + weight*Vec3fa(data.ambient_intensity);
 
     /* calculate transmissivity of hair */
     AnisotropicBlinn brdf;
     float eps = 0.0001f;
 
-    ISPCGeometry* geometry = g_ispc_scene->geometries[ray.geomID];
+    ISPCGeometry* geometry = data.ispc_scene->geometries[ray.geomID];
     if (geometry->type == CURVES)
     {
       /* calculate tangent space */
@@ -309,8 +308,8 @@ Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats&
       const Vec3fa dz = normalize(cross(dy,dx));
 
       /* generate anisotropic BRDF */
-      AnisotropicBlinn__Constructor(&brdf,hair_Kr,hair_Kt,dx,20.0f,dy,2.0f,dz);
-      brdf.Kr = hair_Kr;
+      AnisotropicBlinn__Constructor(&brdf,data.hair_Kr,data.hair_Kt,dx,20.0f,dy,2.0f,dz);
+      brdf.Kr = data.hair_Kr;
     }
     else if (geometry->type == TRIANGLE_MESH)
     {
@@ -328,11 +327,11 @@ Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats&
       return color;
 
     /* sample directional light */
-    Ray shadow(ray.org + ray.tfar*ray.dir, neg(Vec3fa(g_dirlight_direction)), eps, inf, time);
-    Vec3fa T = occluded(g_scene,&context,shadow);
+    Ray shadow(ray.org + ray.tfar*ray.dir, neg(Vec3fa(data.dirlight_direction)), eps, inf, time);
+    Vec3fa T = occluded(data.scene,&context,shadow);
     RayStats_addShadowRay(stats);
-    Vec3fa c = AnisotropicBlinn__eval(&brdf,neg(ray.dir),neg(Vec3fa(g_dirlight_direction)));
-    color = color + weight*c*T*Vec3fa(g_dirlight_intensity);
+    Vec3fa c = AnisotropicBlinn__eval(&brdf,neg(ray.dir),neg(Vec3fa(data.dirlight_direction)));
+    color = color + weight*c*T*Vec3fa(data.dirlight_intensity);
 
 #if 1
     /* sample BRDF */
@@ -370,37 +369,24 @@ Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats&
   return color;
 }
 
-/* renders a single screen tile */
-void renderTileStandard(int taskIndex,
-                        int threadIndex,
-                        int* pixels,
-                        const unsigned int width,
-                        const unsigned int height,
-                        const float time,
-                        const ISPCCamera& camera,
-                        const int numTilesX,
-                        const int numTilesY)
+/* task that renders a single screen tile */
+void renderPixelStandard(const TutorialData& data,
+                         int x, int y, 
+                         int* pixels,
+                         const unsigned int width,
+                         const unsigned int height,
+                         const float time,
+                         const ISPCCamera& camera, RayStats& stats)
 {
-  const unsigned int tileY = taskIndex / numTilesX;
-  const unsigned int tileX = taskIndex - tileY * numTilesX;
-  const unsigned int x0 = tileX * TILE_SIZE_X;
-  const unsigned int x1 = min(x0+TILE_SIZE_X,width);
-  const unsigned int y0 = tileY * TILE_SIZE_Y;
-  const unsigned int y1 = min(y0+TILE_SIZE_Y,height);
+  Vec3fa color = renderPixel(data, (float)x,(float)y,camera,stats);
 
-  for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
-  {
-    /* calculate pixel color */
-    Vec3fa color = renderPixelStandard((float)x,(float)y,camera,g_stats[threadIndex]);
-
-    /* write color to framebuffer */
-    Vec3ff accu_color = g_accu[y*width+x] + Vec3ff(color.x,color.y,color.z,1.0f); g_accu[y*width+x] = accu_color;
-    float f = rcp(max(0.001f,accu_color.w));
-    unsigned int r = (unsigned int) (255.01f * clamp(accu_color.x*f,0.0f,1.0f));
-    unsigned int g = (unsigned int) (255.01f * clamp(accu_color.y*f,0.0f,1.0f));
-    unsigned int b = (unsigned int) (255.01f * clamp(accu_color.z*f,0.0f,1.0f));
-    pixels[y*width+x] = (b << 16) + (g << 8) + r;
-  }
+  /* write color to framebuffer */
+  Vec3ff accu_color = data.accu[y*width+x] + Vec3ff(color.x,color.y,color.z,1.0f); data.accu[y*width+x] = accu_color;
+  float f = rcp(max(0.001f,accu_color.w));
+  unsigned int r = (unsigned int) (255.01f * clamp(accu_color.x*f,0.0f,1.0f));
+  unsigned int g = (unsigned int) (255.01f * clamp(accu_color.y*f,0.0f,1.0f));
+  unsigned int b = (unsigned int) (255.01f * clamp(accu_color.z*f,0.0f,1.0f));
+  pixels[y*width+x] = (b << 16) + (g << 8) + r;
 }
 
 /* task that renders a single screen tile */
@@ -412,26 +398,26 @@ void renderTileTask (int taskIndex, int threadIndex, int* pixels,
                          const int numTilesX,
                          const int numTilesY)
 {
-  renderTileStandard(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
+  const unsigned int tileY = taskIndex / numTilesX;
+  const unsigned int tileX = taskIndex - tileY * numTilesX;
+  const unsigned int x0 = tileX * TILE_SIZE_X;
+  const unsigned int x1 = min(x0+TILE_SIZE_X,width);
+  const unsigned int y0 = tileY * TILE_SIZE_Y;
+  const unsigned int y1 = min(y0+TILE_SIZE_Y,height);
+
+  for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
+  {
+    renderPixelStandard(data,x,y,pixels,width,height,time,camera,g_stats[threadIndex]);
+  }
 }
 
 /* called by the C++ code for initialization */
 extern "C" void device_init (char* cfg)
 {
-  /* initialize last seen camera */
-  g_accu_vx = Vec3fa(0.0f);
-  g_accu_vy = Vec3fa(0.0f);
-  g_accu_vz = Vec3fa(0.0f);
-  g_accu_p  = Vec3fa(0.0f);
-
-  /* initialize hair colors */
-  hair_K  = Vec3fa(0.8f,0.57f,0.32f);
-  hair_dK = Vec3fa(0.1f,0.12f,0.08f);
-  hair_Kr = 0.2f*hair_K;    //!< reflectivity of hair
-  hair_Kt = 0.8f*hair_K;    //!< transparency of hair
-
+  TutorialData_Constructor(&data);
+  
   /* create scene */
-  g_scene = convertScene(g_ispc_scene);
+  g_scene = data.scene = convertScene(data.ispc_scene);
 }
 
 extern "C" void renderFrameStandard (int* pixels,
@@ -441,6 +427,25 @@ extern "C" void renderFrameStandard (int* pixels,
                           const ISPCCamera& camera)
 {
   /* render frame */
+#if defined(EMBREE_SYCL_TUTORIAL) && !defined(EMBREE_SYCL_RT_SIMULATION)
+  TutorialData ldata = data;
+  sycl::event event = global_gpu_queue->submit([=](sycl::handler& cgh){
+    const sycl::nd_range<2> nd_range = make_nd_range(height,width);
+    cgh.parallel_for(nd_range,[=](sycl::nd_item<2> item) {
+      const unsigned int x = item.get_global_id(1); if (x >= width ) return;
+      const unsigned int y = item.get_global_id(0); if (y >= height) return;
+      RayStats stats;
+      renderPixelStandard(ldata,x,y,pixels,width,height,time,camera,stats);
+    });
+  });
+  global_gpu_queue->wait_and_throw();
+
+  const auto t0 = event.template get_profiling_info<sycl::info::event_profiling::command_start>();
+  const auto t1 = event.template get_profiling_info<sycl::info::event_profiling::command_end>();
+  const double dt = (t1-t0)*1E-9;
+  ((ISPCCamera*)&camera)->render_time = dt;
+  
+#else
   const int numTilesX = (width +TILE_SIZE_X-1)/TILE_SIZE_X;
   const int numTilesY = (height+TILE_SIZE_Y-1)/TILE_SIZE_Y;
   parallel_for(size_t(0),size_t(numTilesX*numTilesY),[&](const range<size_t>& range) {
@@ -448,6 +453,7 @@ extern "C" void renderFrameStandard (int* pixels,
     for (size_t i=range.begin(); i<range.end(); i++)
       renderTileTask((int)i,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
   }); 
+#endif
 }
 
 /* called by the C++ code to render */
@@ -458,36 +464,32 @@ extern "C" void device_render (int* pixels,
                            const ISPCCamera& camera)
 {
   /* create accumulator */
-  if (g_accu_width != width || g_accu_height != height) {
-    g_accu = (Vec3ff*) alignedMalloc(width*height*sizeof(Vec3ff),16);
-    g_accu_width = width;
-    g_accu_height = height;
+  if (data.accu_width != width || data.accu_height != height) {
+    data.accu = (Vec3ff*) alignedUSMMalloc((width*height)*sizeof(Vec3ff),16,EMBREE_USM_SHARED_DEVICE_READ_WRITE);
+    data.accu_width = width;
+    data.accu_height = height;
     for (unsigned int i=0; i<width*height; i++)
-      g_accu[i] = Vec3ff(0.0f);
+      data.accu[i] = Vec3ff(0.0f);
   }
 
   /* reset accumulator */
   bool camera_changed = g_changed; g_changed = false;
-  camera_changed |= ne(g_accu_vx,camera.xfm.l.vx); g_accu_vx = camera.xfm.l.vx;
-  camera_changed |= ne(g_accu_vy,camera.xfm.l.vy); g_accu_vy = camera.xfm.l.vy;
-  camera_changed |= ne(g_accu_vz,camera.xfm.l.vz); g_accu_vz = camera.xfm.l.vz;
-  camera_changed |= ne(g_accu_p, camera.xfm.p   ); g_accu_p  = camera.xfm.p;
-  g_accu_count++;
+  camera_changed |= ne(data.accu_vx,camera.xfm.l.vx); data.accu_vx = camera.xfm.l.vx;
+  camera_changed |= ne(data.accu_vy,camera.xfm.l.vy); data.accu_vy = camera.xfm.l.vy;
+  camera_changed |= ne(data.accu_vz,camera.xfm.l.vz); data.accu_vz = camera.xfm.l.vz;
+  camera_changed |= ne(data.accu_p, camera.xfm.p   ); data.accu_p  = camera.xfm.p;
+  data.accu_count++;
   if (camera_changed) {
-    g_accu_count=0;
+    data.accu_count=0;
     for (unsigned int i=0; i<width*height; i++)
-      g_accu[i] = Vec3ff(0.0f);
+      data.accu[i] = Vec3ff(0.0f);
   }
 }
 
 /* called by the C++ code for cleanup */
 extern "C" void device_cleanup ()
 {
-  rtcReleaseScene (g_scene); g_scene = nullptr;
-  alignedFree(g_accu); g_accu = nullptr;
-  g_accu_width = 0;
-  g_accu_height = 0;
-  g_accu_count = 0;
+  TutorialData_Destructor(&data);
 }
 
 } // namespace embree
