@@ -2,6 +2,15 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
+#include "../common/tutorial/benchmark_render.h"
+
+#if defined(EMBREE_SYCL_TUTORIAL)
+#  define NAME "motion_blur_geometry_sycl"
+#  define FEATURES FEATURE_RTCORE | FEATURE_SYCL
+#else
+#  define NAME "motion_blur_geometry"
+#  define FEATURES FEATURE_RTCORE
+#endif
 
 namespace embree
 {
@@ -14,7 +23,7 @@ namespace embree
   struct Tutorial : public TutorialApplication 
   {
     Tutorial()
-      : TutorialApplication("motion_blur_geometry",FEATURE_RTCORE) 
+      : TutorialApplication(NAME,FEATURES) 
     {
       registerOption("time", [] (Ref<ParseStream> cin, const FileName& path) {
         g_time = cin->getFloat();
@@ -38,5 +47,8 @@ namespace embree
 }
 
 int main(int argc, char** argv) {
+  if (embree::TutorialBenchmark::benchmark(argc, argv)) {
+    return embree::TutorialBenchmark(embree::renderBenchFunc<embree::Tutorial>).main(argc, argv, "motion_blur_geometry");
+  }
   return embree::Tutorial().main(argc,argv);
 }
